#include <iostream>

#include <cstdio>
#include <dlfcn.h>
#include <unistd.h>

#include "libfacesdk.h"

//#define TEST_OPEN_TWICE
//#define TEST_CANCEL

// 声明导出函数
typedef int (*CaptureCallBackDetectInfo)(int *nResultID, void *lpParam);
typedef int (*FR_CREATEFACECALLBACK)(CaptureCallBackDetectInfo pDetectInfo, const char *parentDir);
typedef int (*FR_STARTCAMERA)();
typedef int (*FR_STOPCAMERA)();
typedef int (*FR_CREATEWINDOW)(unsigned long ulWndHandle, int iX, int iY, int iWidth, int iHeight);
typedef int (*FR_CLOSEWINDOW)();
typedef int (*FR_STARTLIVEDETECT)(int mode, const char *strContent, const char *strFilePath);
typedef int (*FR_STOPLIVEDETECT)();
typedef int (*FR_FACECOMPARE)(const char *pccPathA, const char *pccPathB, unsigned short ScoreIn, unsigned short &ScoreOut);
typedef int (*FR_PAUSEANDPLAY)(bool isPause);
typedef int (*FR_HIDETIPS)(bool isHide);
typedef void (*FR_GETSTATUS)(int *piStatus);
typedef int (*FR_DESTROYFACECALLBACK)();
typedef int (*FR_CROPIMAGE)(const char*pccPathA, int crop_width, int crop_height, const char*pccPathB);
typedef int (*FR_COMPRESSIMAGE)(const char*pccPathA, int max_size, const char*pccPathB);


FR_CREATEFACECALLBACK fr_CreateFaceCallBack = NULL;
FR_STARTCAMERA fr_StartCamera = NULL;
FR_STOPCAMERA fr_StopCamera = NULL;
FR_CREATEWINDOW fr_CreateWindow = NULL;
FR_CLOSEWINDOW fr_CloseWindow = NULL;
FR_STARTLIVEDETECT fr_StartLiveDetect = NULL;
FR_STOPLIVEDETECT fr_StopLiveDetect = NULL;
FR_FACECOMPARE fr_FaceCompare = NULL;
FR_PAUSEANDPLAY fr_PauseAndPlay = NULL;
FR_HIDETIPS fr_HideTips = NULL;
FR_GETSTATUS fr_GetStatus = NULL;
FR_DESTROYFACECALLBACK fr_DestroyFaceCallback = NULL;
FR_CROPIMAGE fr_CropImage = NULL;
FR_COMPRESSIMAGE fr_CompressImage = NULL;

enum FaceDetectStatus 
{
	FD_STATUS_DETECTING = 1, // 检测中
	FD_STATUS_FAILED = 2, // 失败
	FD_STATUS_SUCCESS = 3 // 成功
};

int g_detect_status = 0; // 人脸检活状态  FaceDetectStatus

// 加载SDK库并导出函数
bool load_sdk()
{
	void *handler;

	handler = dlopen("../EyeCool_CAM/libfacesdk.so", RTLD_LAZY);
	if (handler == NULL)
	{
		std::cout << "dlopen failed!" << std::endl;
		return false;
	}

	fr_CreateFaceCallBack = (FR_CREATEFACECALLBACK)dlsym(handler, "FR_CreateFaceCallBack");
	fr_StartCamera = (FR_STARTCAMERA)dlsym(handler, "FR_StartCamera");
	fr_StopCamera = (FR_STOPCAMERA)dlsym(handler, "FR_StopCamera");
	fr_CreateWindow = (FR_CREATEWINDOW)dlsym(handler, "FR_CreateWindow");
	fr_CloseWindow = (FR_CLOSEWINDOW)dlsym(handler, "FR_CloseWindow");
	fr_StartLiveDetect = (FR_STARTLIVEDETECT)dlsym(handler, "FR_StartLiveDetect");
	fr_StopLiveDetect = (FR_STOPLIVEDETECT)dlsym(handler, "FR_StopLiveDetect");
	fr_FaceCompare = (FR_FACECOMPARE)dlsym(handler, "FR_FaceCompare");
	fr_PauseAndPlay = (FR_PAUSEANDPLAY)dlsym(handler, "FR_PauseAndPlay");
	fr_HideTips = (FR_HIDETIPS)dlsym(handler, "FR_HideTips");
	fr_GetStatus = (FR_GETSTATUS)dlsym(handler, "FR_GetStatus");
	fr_DestroyFaceCallback = (FR_DESTROYFACECALLBACK)dlsym(handler, "FR_DestroyFaceCallBack");
	fr_CropImage = (FR_CROPIMAGE)dlsym(handler, "FR_CropImage");
	fr_CompressImage = (FR_COMPRESSIMAGE)dlsym(handler, "FR_CompressImage");

	if (fr_CreateFaceCallBack == NULL || fr_StartCamera == NULL || fr_StopCamera == NULL || fr_CreateWindow == NULL ||
		fr_CloseWindow == NULL || fr_StartLiveDetect == NULL || fr_StopLiveDetect == NULL || fr_FaceCompare == NULL ||
		fr_PauseAndPlay == NULL || fr_HideTips == NULL || fr_GetStatus == NULL || fr_DestroyFaceCallback == NULL ||
		fr_CropImage == NULL || fr_CompressImage == NULL)
	{
		std::cout << "export function failed" << std::endl;
		return false;
	}

	return true;
}

// 人脸检测返回事件的回调函数
int face_detect_event_cb(int *nResultID, void *lpParam)
{
	std::cout << "nResultID = " << *nResultID << std::endl;

	// 采集人脸图像并活体检验成功
	if (*nResultID == FR_CALLBACK_RESULT_SUCC)
	{
		g_detect_status = FD_STATUS_SUCCESS;
	}
	else if (*nResultID == FR_CALLBACK_RESULT_FAIL || 
			 *nResultID == FR_CALLBACK_RESULT_ERROR || 
			 *nResultID == FR_CALLBACK_RESULT_TIMEOUT ||
			 *nResultID == FR_CALLBACK_RESULT_CANCEL ||
			 *nResultID == FR_CALLBACK_RESULT_UNKNOW ) 
	{
		g_detect_status = FD_STATUS_FAILED;
	}
	else 
	{
		g_detect_status = FD_STATUS_DETECTING;	
	}

	return 0;
}

int main()
{
	if (!load_sdk())
	{
		std::cout << "load_sdk failed!" << std::endl;
		return -1;
	}

	// 初始化SDK状态，并设置检活回调函数。 该函数必须首先调用。
	int ret = fr_CreateFaceCallBack(face_detect_event_cb, "..");
	if (ret != FR_RET_SUCC)
	{
		std::cout << "FR_CreateFaceCallBack ret = " << ret << std::endl;
		return -1;
	}

	char c;

#ifdef TEST_OPEN_TWICE
	int cap_cnt = 0;
#else
	int cap_cnt = 1;
#endif

	do
	{
		// 查看摄像头状态
		int status;
		fr_GetStatus(&status);
		std::cout << "camera status = " << status << std::endl;

		// 打开摄像头
		ret = fr_StartCamera();
		if (ret != FR_RET_SUCC)
		{
			std::cout << "FR_StartCamera ret = " << ret << std::endl;
			return -1;
		}
		std::cout << "StartCamera OK " << std::endl;

		// 创建预览窗口
		if(cap_cnt % 2 == 0)
		{
			ret = fr_CreateWindow(0, 100, 100, 640, 480);
		}
		else
		{
			ret = fr_CreateWindow(0, 300, 100, 600, 600);
		}
		if (ret != FR_RET_SUCC)
		{
			std::cout << "FR_CreateWindow ret = " << ret << std::endl;
			return -1;
		}
		std::cout << "CreateWindow OK " << std::endl;

		std::cout << "Start Face Live Detect" << std::endl;

		char file[128] = {0};
		sprintf(file, "living.jpg");
		g_detect_status = FD_STATUS_DETECTING;
		ret = fr_StartLiveDetect(1, "", file);
		if (ret != FR_RET_SUCC)
		{
			std::cout << "FR_StartLiveDetect = " << ret << std::endl;
			return -1;
		}

		int wait_cnt = 0;
		while (g_detect_status == FD_STATUS_DETECTING)
		{
#ifdef TEST_CANCEL
			if(++wait_cnt == 5)
			{
				ret = fr_StopLiveDetect();
				std::cout << "FR_StopLiveDetect = " << ret << std::endl;
			}
#endif

			usleep(200);
		}

		// !!! sample只是演示常规的调用流程，为了简单，将采集的照片和自身进行比较验证比对功能是否正常，
		// 实际应用，可以将比较的两张图片替换为一个是存储的模板图片，一个是现场采集的图片进行比较。
		if (g_detect_status == FD_STATUS_SUCCESS)
		{
			std::cout << "Face Detect Success" << std::endl;

			ret = fr_CropImage("../EyeCool_CAM/living.jpg", 300, 400, "../EyeCool_CAM/crop_living.jpg");
			if (ret == FR_RET_SUCC)
			{
				std::cout << "crop face image success" << std::endl;
			}
			else
			{
				std::cout << "crop face image failed" << std::endl;
			}

			ret = fr_CompressImage("../EyeCool_CAM/crop_living.jpg", 30, "../EyeCool_CAM/compress_living.jpg");
			if (ret == FR_RET_SUCC)
			{
				std::cout << "compress face image success" << std::endl;
			}
			else
			{
				std::cout << "compress face image failed" << std::endl;
			}

			unsigned short in = 60;
			unsigned short score = 0;
			ret = fr_FaceCompare("../EyeCool_CAM/living.jpg", "../EyeCool_CAM/living.jpg", in, score);
			std::cout << "FR_FaceCompare = " << ret << std::endl;
			if (ret == FR_RET_SUCC)
			{
				std::cout << "face compare success" << std::endl;
			}
			else
			{
				std::cout << "face compare failed" << std::endl;
			}
			std::cout << "score = " << score << std::endl;
		}
		else if (g_detect_status == FD_STATUS_FAILED)
		{
			std::cout << "Face Detect Failed" << std::endl;
		}

		ret = fr_CloseWindow ();
		if (ret != FR_RET_SUCC)
		{
			std::cout << "fr_CloseWindow  failed" << std::endl;
		}
		ret = fr_StopCamera();
		if (ret != FR_RET_SUCC)
		{
			std::cout << "StopCamera failed" << std::endl;
		}

		++cap_cnt;
	}
	while(cap_cnt < 2);
	//while((c = getchar()) != 'q');

	ret = fr_DestroyFaceCallback();
	if (ret != FR_RET_SUCC)
	{
		std::cout << "fr_DestroyFaceCallback = " << ret << std::endl;
	}

	return 0;
}
